@@ -1,4 +1,4 @@
-# Copyright 2014-2020 Richard Dymond (rjdymond@gmail.com)
+# Copyright 2014-2022 Richard Dymond (rjdymond@gmail.com)
 #
 # This file is part of SkoolKit.
 #
@@ -19,6 +19,17 @@ import re
 
 SECTIONS = OrderedDict()
 
+SECTIONS['AudioWriter'] = """
+; ClockSpeed=3500000
+; ContentionBegin=14334
+; ContentionEnd=57248
+; ContentionFactor=51
+; FrameDuration=69888
+; InterruptDelay=942
+; MaxAmplitude=65536
+; SampleRate=44100
+"""
+
 SECTIONS['Colours'] = """
 ; TRANSPARENT=0,254,0
 ; BLACK=0,0,0
@@ -39,14 +50,18 @@ SECTIONS['Colours'] = """
 """
 
 SECTIONS['Config'] = """
+; Expand=
 ; GameDir=
 HtmlWriterClass=skoolkit.skoolhtml.HtmlWriter
+; InitModule=
 ; RefFiles=
 """
 
 SECTIONS['Game'] = """
+Address=
 AddressAnchor={address}
 AsmSinglePage=0
+AudioFormats=flac,mp3,ogg
 Bytes=
 Copyright=
 Created=Created using <a href="https://skoolkit.ca">SkoolKit</a> #VERSION.
@@ -55,6 +70,7 @@ DisassemblyTableNumCols=5
 ; Game=
 InputRegisterTableHeader=Input
 ; JavaScript=
+Length={size}
 LinkInternalOperands=0
 LinkOperands=CALL,DEFW,DJNZ,JP,JR
 ; Logo=
@@ -119,7 +135,7 @@ MemoryMap=Everything
 
 SECTIONS['MemoryMap:MemoryMap'] = """
 ; EntryDescriptions=0
-; EntryTypes=bcgstuw
+EntryTypes=bcgstuw
 ; Includes=
 ; Intro=
 ; LabelColumn=0
@@ -232,6 +248,7 @@ GameIndex=The complete<>RAM disassembly
 """
 
 SECTIONS['Paths'] = """
+AudioPath=audio
 CodePath=asm
 FontPath=.
 FontImagePath={ImagePath}/font
@@ -493,6 +510,12 @@ SECTIONS['Template:asm_single_page'] = """
 <# endfor #>
 """
 
+SECTIONS['Template:audio'] = """
+<audio controls="" src="{src}">
+<p>Your browser doesn't support HTML5 audio. Here is a <a href="{src}">link to the audio</a> instead.</p>
+</audio>
+"""
+
 SECTIONS['Template:box_entries'] = """
 <ul class="contents">
 <# foreach($entry,entries) #>
@@ -605,7 +628,7 @@ SECTIONS['Template:memory_map'] = """
 <# endif #>
 <td class="map-{$entry[type]}"><span id="{$entry[anchor]}"></span><a href="{$entry[href]}">{$entry[address]}</a></td>
 <# if({MemoryMap[LengthColumn]}) #>
-<td class="map-length">{$entry[size]}</td>
+<td class="map-length">{$entry[length]}</td>
 <# endif #>
 <td class="map-{$entry[type]}-desc">
 <div class="map-entry-title-1{MemoryMap[EntryDescriptions]}"><a class="map-entry-title" href="{$entry[href]}">{$entry[title]}</a></div>
