@@ -1,4 +1,4 @@
-# Copyright 2010-2020 Richard Dymond (rjdymond@gmail.com)
+# Copyright 2010-2022 Richard Dymond (rjdymond@gmail.com)
 #
 # This file is part of SkoolKit.
 #
@@ -41,7 +41,7 @@ AD_REFS = 'refs'
 
 # An entry ASM directive is one that should be placed before the entry title
 # when it is associated with the first instruction in the entry
-RE_ENTRY_ASM_DIRECTIVE = re.compile("assemble=|def[bsw]=|end$|equ=|expand=|if\(|org$|org=|remote=|replace=|set-[-a-z]+=|start$|writer=")
+RE_ENTRY_ASM_DIRECTIVE = re.compile("assemble=|def[bsw]=|end$|equ=|expand=|if\(|org$|org=|remote=|replace=|rom|set-[-a-z]+=|start$|writer=")
 
 # Comment types to which the @ignoreua directive may be applied
 TITLE = 't'
@@ -74,10 +74,12 @@ class ControlDirectiveComposer:
                           hexadecimal values with explicit 'd' and 'h' base
                           indicators.
     """
+    # Component API
     def __init__(self, preserve_base):
         self.preserve_base = preserve_base
         self.op_evaluator = get_operand_evaluator()
 
+    # Component API
     def compose(self, operation):
         """Compute the type, length and sublengths of a DEFB/DEFM/DEFS/DEFW
         statement, or the operand bases of a regular instruction.
