@@ -1,4 +1,4 @@
-# Copyright 2008-2020 Richard Dymond (rjdymond@gmail.com)
+# Copyright 2008-2022 Richard Dymond (rjdymond@gmail.com)
 #
 # This file is part of SkoolKit.
 #
@@ -26,11 +26,11 @@ from collections import defaultdict
 import re
 from io import StringIO
 
-from skoolkit import skoolmacro, SkoolKitError, evaluate, format_template, parse_int, warn
-from skoolkit.components import get_component
+from skoolkit import skoolmacro, SkoolKitError, SkoolParsingError, evaluate, format_template, parse_int, warn
+from skoolkit.audio import AudioWriter
+from skoolkit.components import get_component, get_image_writer
 from skoolkit.defaults import REF_FILE
 from skoolkit.graphics import Frame, adjust_udgs, build_udg, font_udgs, scr_udgs
-from skoolkit.image import ImageWriter
 from skoolkit.refparser import RefParser
 from skoolkit.skoolparser import TableParser, ListParser
 
@@ -47,9 +47,6 @@ T_LAYOUT = 'Layout'
 # UDG image path ID
 UDG_IMAGE_PATH = 'UDGImagePath'
 
-# Default memory map entry types
-DEF_MEMORY_MAP_ENTRY_TYPES = 'bcgstuw'
-
 def join(*path_components):
     return '/'.join([c for c in path_components if c.replace('/', '')])
 
@@ -72,11 +69,9 @@ class HtmlWriter:
         self.defaults.parse(StringIO(REF_FILE))
         self.file_info = file_info
 
-        self.fields = skool_parser.fields
-
         colours = self._parse_colours(self.get_dictionary('Colours'))
-        iw_options = self.get_dictionary('ImageWriter')
-        self.image_writer = ImageWriter(colours, iw_options)
+        iw_config = self.get_dictionary('ImageWriter')
+        self.image_writer = get_image_writer(iw_config, colours)
         self.frames = {}
 
         self.snapshot = self.parser.snapshot
@@ -88,22 +83,31 @@ class HtmlWriter:
 
         self.table_parser = TableParser()
         self.list_parser = ListParser()
-        self.to_chr = lambda n: '&#{};'.format(n)
+        self.to_chr = '&#{};'.format
         self.get_reg = lambda r: self.format_template('reg', {'reg': r})
         self.space = '&#160;'
         self.pc = 0
         self.macros = skoolmacro.get_macros(self)
-        for e in self.parser.expands:
-            self.expand(e)
+        for m in self.parser.expands + [ref_parser.get_dictionary('Config').get('Expand', '')]:
+            try:
+                self.expand(m)
+            except SkoolParsingError as e:
+                raise SkoolParsingError("Failed to expand '{}': {}".format(m, e.args[0]))
+            except:
+                raise SkoolKitError("Failed to expand '{}'".format(m))
 
         self.game_vars = self._expand_values('Game', 'Logo')
         self.asm_anchor_template = self.game_vars['AddressAnchor']
+        self.asm_address_template = self.game_vars['Address']
         self.asm_single_page = self.game_vars['AsmSinglePage'] != '0'
         self.paths = self.get_dictionary('Paths')
         self.titles = self.get_dictionary('Titles')
         self.page_headers = self.get_dictionary('PageHeaders')
         links = self.get_dictionary('Links')
 
+        self.audio_writer = AudioWriter(self.get_dictionary('AudioWriter'))
+        self.audio_formats = ['.' + f for f in self.game_vars['AudioFormats'].split(',')]
+
         self.page_ids = []
         self.pages = defaultdict(dict)
         self.box_pages = {}
@@ -145,6 +149,7 @@ class HtmlWriter:
             self.paths.setdefault(page_id, path)
             self.titles.setdefault(page_id, page_id)
 
+        self.memory_maps = {}
         self.other_code = []
         other_code_indexes = set()
         for c_id, code in self.get_dictionaries('OtherCode'):
@@ -152,6 +157,7 @@ class HtmlWriter:
             self.other_code.append((c_id, code))
             index_page_id = code['IndexPageId'] = '{}-Index'.format(c_id)
             other_code_indexes.add(index_page_id)
+            self.memory_maps[index_page_id] = {'EntryTypes': 'bcgstuw'}
             self.paths.setdefault(index_page_id, '{0}/{0}.html'.format(c_id))
             self.titles.setdefault(index_page_id, c_id)
             code_path_id = code['CodePathId'] = '{}-CodePath'.format(c_id)
@@ -166,16 +172,21 @@ class HtmlWriter:
                     self.titles.setdefault(asm_page_id, self.titles[default_asm_page_id])
                     self.page_headers.setdefault(asm_page_id, self.page_headers[default_asm_page_id])
 
+        self.entry_groups = {}
+        group_entries = defaultdict(set)
+        for group, addresses in self.get_dictionary('EntryGroups').items():
+            for spec in addresses.split(','):
+                for address in self._parse_addresses(spec):
+                    self.entry_groups[address] = 'Asm-' + group
+                    group_entries[group].add(address)
+
         self.main_memory_maps = []
-        self.memory_maps = {}
         for map_name, map_details in self.get_dictionaries('MemoryMap'):
             self._expand_values(map_details, 'Intro')
             self.memory_maps[map_name] = map_details
             includes = []
-            for addr_str in map_details.get('Includes', '').split(','):
-                address = parse_int(addr_str)
-                if self.get_entry(address):
-                    includes.append(address)
+            for spec in map_details.get('Includes', '').split(','):
+                includes.extend(self._parse_addresses(spec, group_entries))
             map_details['Includes'] = includes
             if map_name not in other_code_indexes and self._should_write_map(map_details):
                 self.main_memory_maps.append(map_name)
@@ -240,6 +251,11 @@ class HtmlWriter:
     def case(self):
         return self.parser.case
 
+    # API
+    @property
+    def fields(self):
+        return self.parser.fields
+
     def warn(self, s):
         warn(s)
 
@@ -315,6 +331,15 @@ class HtmlWriter:
                 new_links[page_id] = (link_text.strip(), '')
         return new_links
 
+    def _parse_addresses(self, spec, groups=None):
+        limits = spec.partition('-')[::2]
+        start = parse_int(limits[0])
+        if start is None:
+            return (groups or {}).get(limits[0], ())
+        start = max(start, self.memory_map[0].address)
+        end = min(parse_int(limits[1], start), self.memory_map[-1].address)
+        return [a for a in range(start, end + 1) if self.get_entry(a)]
+
     def get_code_path(self, code_id):
         if code_id.lower() == MAIN_CODE_ID:
             return self.paths['CodePath']
@@ -462,6 +487,11 @@ class HtmlWriter:
             return 'RAW(#{})'.format(anchor)
         return anchor
 
+    def asm_address(self, address, default):
+        if self.asm_address_template:
+            return self.asm_address_template.format(address=address)
+        return default
+
     # API
     def screenshot(self, x=0, y=0, w=32, h=24, df_addr=16384, af_addr=22528):
         """Return a two-dimensional array of tiles (instances of
@@ -531,7 +561,7 @@ class HtmlWriter:
             'exists': 1,
             'type': entry.ctl,
             'location': entry.address,
-            'address': entry.addr_str,
+            'address': self.asm_address(entry.address, entry.addr_str),
             'anchor': self.asm_anchor(entry.address),
             'page': entry.address // 256,
             'byte': entry.address % 256,
@@ -539,6 +569,7 @@ class HtmlWriter:
             'description': description,
             'href': self._asm_relpath(cwd, entry.address),
             'size': entry.size,
+            'length': self.game_vars['Length'].format(size=entry.size),
             'title': self.expand(entry.description, cwd)
         }
 
@@ -697,7 +728,7 @@ class HtmlWriter:
 
             instructions.append({
                 'block_comment': block_comment,
-                'address': instruction.addr_str,
+                'address': self.asm_address(instruction.address, instruction.addr_str),
                 'location': instruction.address,
                 'called': 1 + int(instruction.ctl in 'c*'),
                 'label': instruction.asm_label or '',
@@ -719,7 +750,8 @@ class HtmlWriter:
         entry = self.memory_map[index]
         page_id = self._get_asm_page_id(self.code_id, entry.ctl)
         fname = join(cwd, self.asm_fname(entry.address))
-        self._set_cwd(page_id, 'asm', fname)
+        group = self.entry_groups.get(entry.address)
+        self._set_cwd(page_id, 'asm', fname, group)
 
         subs = {'entry': self._get_asm_entry(cwd, index, map_file)}
         self.skoolkit['title'] = self.skoolkit['title'].format(**subs)
@@ -762,14 +794,14 @@ class HtmlWriter:
             return False
         if map_details['Includes']:
             return True
-        entry_types = map_details.get('EntryTypes', DEF_MEMORY_MAP_ENTRY_TYPES)
+        entry_types = map_details.get('EntryTypes', '')
         return any([entry.ctl in entry_types for entry in self.memory_map])
 
     def write_map(self, map_name):
         fname, cwd = self._set_cwd(map_name, 'memory_map')
 
         map_details = self.memory_maps.get(map_name, {})
-        entry_types = map_details.get('EntryTypes', DEF_MEMORY_MAP_ENTRY_TYPES)
+        entry_types = map_details.get('EntryTypes', '')
         map_dict = {
             'EntryDescriptions': map_details.get('EntryDescriptions', '0'),
             'EntryTypes': entry_types,
@@ -820,7 +852,7 @@ class HtmlWriter:
         with self.file_info.open_file(fname) as f:
             f.write(contents)
 
-    def _set_cwd(self, page_id, include, asm_fname=None, js=None):
+    def _set_cwd(self, page_id, include, asm_fname=None, group=None, js=None):
         if asm_fname is None:
             fname = self.paths[page_id]
         else:
@@ -838,11 +870,14 @@ class HtmlWriter:
                 js_files += tuple(js.split(';'))
             self.javascript[js_key] = [{'src': self.relpath(cwd, join(self.paths['JavaScriptPath'], basename(j)))} for j in js_files]
 
+        title = self.titles.get(group, self.titles[page_id])
+        page_header = self.page_headers.get(group, self.page_headers[page_id])
+
         self.skoolkit['page_id'] = page_id
         self.skoolkit['path'] = fname
         self.skoolkit['index_href'] = self.relpath(cwd, self.paths[P_GAME_INDEX])
-        self.skoolkit['title'] = self.expand(self.titles[page_id], cwd)
-        self.skoolkit['page_header'] = self.expand(self.page_headers[page_id], cwd).rpartition('<>')[::2]
+        self.skoolkit['title'] = self.expand(title, cwd)
+        self.skoolkit['page_header'] = self.expand(page_header, cwd).rpartition('<>')[::2]
         self.skoolkit['include'] = include
         self.skoolkit['javascripts'] = self.javascript[js_key]
         self.skoolkit['stylesheets'] = self.stylesheets[cwd]
@@ -910,17 +945,26 @@ class HtmlWriter:
         image_path = self._image_path(fname, path_id)
         if image_path:
             if self.file_info.need_image(image_path):
-                self._write_image(image_path, frames)
-            if alt is None:
-                alt = basename(image_path)[:-4]
-            return self.format_img(alt, self.relpath(cwd, image_path))
+                content = self._write_image(image_path, frames)
+            else:
+                content = None
+            if content is None:
+                if alt is None:
+                    alt = basename(image_path)[:-4]
+                return self.format_img(alt, self.relpath(cwd, image_path))
+            return content
         return ''
 
     def _write_image(self, image_path, frames):
         f = self.file_info.open_file(image_path, mode='wb')
-        self.image_writer.write_image(frames, f)
+        content = self.image_writer.write_image(frames, f)
+        fsize = f.tell()
         f.close()
-        self.file_info.add_image(image_path)
+        if fsize:
+            self.file_info.add_image(image_path)
+        elif isfile(f.name):
+            os.remove(f.name)
+        return content
 
     def build_table(self, table):
         rows = []
@@ -961,15 +1005,12 @@ class HtmlWriter:
                         field.
         """
         if fname:
+            fname = self.image_writer.image_fname(fname)
             expanded = self._expand_image_path(fname)
-            if expanded[-4:].lower() == '.png':
-                suffix = ''
-            else:
-                suffix = '.png'
             if expanded != fname or fname.startswith('/'):
-                return expanded.lstrip('/') + suffix
+                return expanded.lstrip('/')
             if path_id in self.paths:
-                return join(self._expand_image_path(self.paths[path_id]), fname + suffix)
+                return join(self._expand_image_path(self.paths[path_id]), fname)
             raise SkoolKitError("Unknown path ID '{0}' for image file '{1}'".format(path_id, fname))
 
     def _expand_image_path(self, path):
@@ -984,8 +1025,38 @@ class HtmlWriter:
             prev_path = path
         return path
 
+    def _write_audio(self, audio_path, delays, flags, offset):
+        f = self.file_info.open_file(audio_path, mode='wb')
+        self.audio_writer.write_audio(f, delays, flags & 1, flags & 2, offset)
+        f.close()
+        self.file_info.add_audio(audio_path)
+
+    def _need_audio(self, fname):
+        if fname.startswith('/'):
+            fname = fname.lstrip('/')
+        else:
+            fname = join(self.paths['AudioPath'], fname)
+        for fmt in self.audio_writer.formats():
+            if fname.lower().endswith(fmt.lower()):
+                bname = fname[:-len(fmt)]
+                for alt_fmt in self.audio_formats:
+                    alt_fname = bname + alt_fmt
+                    if self.file_info.file_exists(alt_fname):
+                        return alt_fname, False
+                return fname, self.file_info.need_audio(fname)
+        return fname, False
+
+    def expand_audio(self, text, index, cwd):
+        end, flags, offset, fname, delays = skoolmacro.parse_audio(self, text, index, self._need_audio)
+        if delays:
+            self._write_audio(fname, delays, flags, offset)
+        return end, self.format_template('audio', {'src': self.relpath(cwd, fname)})
+
+    def expand_copy(self, text, index, cwd):
+        return skoolmacro.parse_copy(text, index, self.fields, self.frames)
+
     def expand_font(self, text, index, cwd):
-        end, crop_rect, fname, frame, alt, params = skoolmacro.parse_font(text, index)
+        end, crop_rect, fname, frame, alt, params = skoolmacro.parse_font(text, index, self.fields)
         message, addr, chars, attr, scale, tindex, alpha = params
         udgs = lambda: font_udgs(self.snapshot, addr, attr, unescape(message)[:chars])
         frame = Frame(udgs, scale, 0, *crop_rect, name=frame, tindex=tindex, alpha=alpha)
@@ -996,11 +1067,14 @@ class HtmlWriter:
         return end, unescape(content)
 
     def expand_include(self, text, index, cwd):
-        end, paragraphs, section = skoolmacro.parse_include(text, index)
-        content = self.get_section(section, paragraphs)
+        end, paragraphs, pattern = skoolmacro.parse_include(text, index, self.fields)
+        try:
+            content = self.ref_parser.combine_sections(pattern, paragraphs)
+        except re.error as e:
+            raise SkoolParsingError(f"Error while parsing #INCLUDE macro: {e.msg}: '{e.pattern}'")
         if paragraphs:
             return end, self.format_template('section', {'section': [self.expand(p, cwd).strip() for p in content]})
-        return end, content
+        return end, '\n'.join(content)
 
     def expand_link(self, text, index, cwd):
         end, page_id, anchor, link_text = skoolmacro.parse_link(text, index)
@@ -1025,13 +1099,19 @@ class HtmlWriter:
     def expand_list(self, text, index, cwd):
         # #LIST[(class)]<items>LIST#
         end, list_obj = self.list_parser.parse_text(self, text, index, cwd)
-        return end, self.build_list(list_obj)
+        return -end, self.build_list(list_obj)
+
+    def expand_over(self, text, index, cwd):
+        return skoolmacro.parse_over(text, index, self.fields, self.frames)
+
+    def expand_plot(self, text, index, cwd):
+        return skoolmacro.parse_plot(text, index, self.fields, self.frames)
 
     def expand_r(self, text, index, cwd):
-        end, addr_str, address, code_id, anchor, link_text = skoolmacro.parse_r(text, index)
+        end, addr_str, address, code_id, anchor, link_text = skoolmacro.parse_r(self.fields, text, index)
         container = self.parser.get_container(address, code_id)
         if (not code_id or code_id == self.code_id) and not container:
-            raise skoolmacro.MacroParsingError('Could not find instruction at {}'.format(addr_str))
+            raise skoolmacro.MacroParsingError(f'Address not found: {addr_str}')
         if self.asm_single_page:
             href = self._asm_relpath(cwd, address, code_id, True)
         else:
@@ -1050,10 +1130,10 @@ class HtmlWriter:
             href = self._asm_relpath(cwd, container_address, code_id) + anchor
         asm_label = self.parser.get_asm_label(address)
         inst_addr_str = self.parser.get_instruction_addr_str(address, addr_str, code_id)
-        return end, self.format_link(href, link_text or asm_label or inst_addr_str)
+        return end, self.format_link(href, link_text or asm_label or self.asm_address(address, inst_addr_str))
 
     def expand_scr(self, text, index, cwd):
-        end, crop_rect, fname, frame, alt, params = skoolmacro.parse_scr(text, index)
+        end, crop_rect, fname, frame, alt, params = skoolmacro.parse_scr(text, index, self.fields)
         scale, x, y, w, h, df, af, tindex, alpha = params
         udgs = lambda: self.screenshot(x, y, w, h, df, af)
         frame = Frame(udgs, scale, 0, *crop_rect, name=frame, tindex=tindex, alpha=alpha)
@@ -1062,10 +1142,10 @@ class HtmlWriter:
     def expand_table(self, text, index, cwd):
         # #TABLE[(class[,col1class[,col2class...]])]<rows>TABLE#
         end, table = self.table_parser.parse_text(self, text, index, cwd)
-        return end, self.build_table(table)
+        return -end, self.build_table(table)
 
     def expand_udg(self, text, index, cwd):
-        end, crop_rect, fname, frame, alt, params = skoolmacro.parse_udg(text, index)
+        end, crop_rect, fname, frame, alt, params = skoolmacro.parse_udg(text, index, self.fields)
         addr, attr, scale, step, inc, flip, rotate, mask, tindex, alpha, mask_addr, mask_step = params
         udgs = lambda: [[build_udg(self.snapshot, addr, attr, step, inc, flip, rotate, mask, mask_addr, mask_step)]]
         if not fname and not frame:
@@ -1076,14 +1156,21 @@ class HtmlWriter:
         return end, self.handle_image(frame, fname, cwd, alt, UDG_IMAGE_PATH)
 
     def _expand_udgarray_with_frames(self, text, index, cwd):
-        end, fname, alt, frames = skoolmacro.parse_udgarray_with_frames(text, index, self.frames)
+        end, fname, alt, frames = skoolmacro.parse_udgarray_with_frames(text, index, self.fields, self.frames)
         return end, self.handle_image(frames, fname, cwd, alt, UDG_IMAGE_PATH)
 
     def expand_udgarray(self, text, index, cwd):
         if index < len(text) and text[index] == '*':
             return self._expand_udgarray_with_frames(text, index, cwd)
 
-        end, crop_rect, fname, frame, alt, params = skoolmacro.parse_udgarray(text, index, self.snapshot)
+        end, crop_rect, fname, frame, alt, params = skoolmacro.parse_udgarray(text, index, self.snapshot, fields=self.fields)
+        udg_array, scale, flip, rotate, mask, tindex, alpha = params
+        udgs = lambda: adjust_udgs(udg_array, flip, rotate)
+        frame = Frame(udgs, scale, mask, *crop_rect, name=frame, tindex=tindex, alpha=alpha)
+        return end, self.handle_image(frame, fname, cwd, alt, UDG_IMAGE_PATH)
+
+    def expand_udgs(self, text, index, cwd):
+        end, crop_rect, fname, frame, alt, params = skoolmacro.parse_udgs(self, text, index, cwd)
         udg_array, scale, flip, rotate, mask, tindex, alpha = params
         udgs = lambda: adjust_udgs(udg_array, flip, rotate)
         frame = Frame(udgs, scale, mask, *crop_rect, name=frame, tindex=tindex, alpha=alpha)
@@ -1105,13 +1192,16 @@ class FileInfo:
 
     :param topdir: The top-level directory.
     :param game_dir: The subdirectory of `topdir` in which to write all HTML
-                     files and image files.
-    :param replace_images: Whether existing images should be overwritten.
+                     files, image files and audio files.
+    :param replace_images: Whether to overwrite existing image files.
+    :param replace_audio: Whether to overwrite existing audio files.
     """
-    def __init__(self, topdir, game_dir, replace_images):
+    def __init__(self, topdir, game_dir, replace_images, replace_audio):
         self.odir = join(topdir, game_dir)
         self.replace_images = replace_images
         self.images = set()
+        self.replace_audio = replace_audio
+        self.audio = set()
 
     def open_file(self, *names, mode='w'):
         path = self.odir
@@ -1127,6 +1217,14 @@ class FileInfo:
     def need_image(self, image_path):
         return image_path not in self.images if self.replace_images else not self.file_exists(image_path)
 
+    def add_audio(self, audio_path):
+        self.audio.add(audio_path)
+
+    def need_audio(self, audio_path):
+        if self.replace_audio:
+            return audio_path not in self.audio
+        return not self.file_exists(audio_path)
+
     def file_exists(self, fname):
         return isfile(join(self.odir, fname))
 
@@ -1148,9 +1246,11 @@ class TemplateFormatter:
 
     :param templates: A dictionary of templates keyed by template name.
     """
+    # Component API
     def __init__(self, templates):
         self.templates = templates
 
+    # Component API
     def format_template(self, page_id, name, fields):
         """Format a template.
 
