@@ -1,4 +1,4 @@
-# Copyright 2008-2020 Richard Dymond (rjdymond@gmail.com)
+# Copyright 2008-2022 Richard Dymond (rjdymond@gmail.com)
 #
 # This file is part of SkoolKit.
 #
@@ -18,8 +18,8 @@ from collections import defaultdict, namedtuple
 from html import escape
 import re
 
-from skoolkit import (BASE_10, BASE_16, CASE_LOWER, CASE_UPPER, SkoolParsingError,
-                      warn, wrap, get_int_param, parse_int, open_file, z80)
+from skoolkit import (BASE_10, BASE_16, CASE_LOWER, CASE_UPPER, ROM48, SkoolParsingError,
+                      warn, wrap, get_int_param, parse_int, read_bin_file, open_file, z80)
 from skoolkit.components import get_assembler, get_instruction_utility
 from skoolkit.skoolmacro import INTEGER, ClosingBracketError, MacroParsingError, parse_brackets, parse_if, parse_strings
 from skoolkit.textutils import partition_unquoted, split_quoted, split_unquoted
@@ -410,6 +410,9 @@ class SkoolParser:
     :param create_labels: Whether to create default labels for unlabelled
                           instructions.
     :param asm_labels: Whether to parse `@label` directives.
+    :param label_fmt: A 2-element tuple containing the format of the default
+                      label for an entry, and the format of the default label
+                      for an entry point.
     :param min_address: Ignore addresses below this one.
     :param max_address: Ignore addresses above this one.
     :param variables: List of (name, value) tuples defining variables that are
@@ -419,12 +422,13 @@ class SkoolParser:
     :param expands: List of @expand directive values.
     """
     def __init__(self, skoolfile, case=0, base=0, asm_mode=0, warnings=False, fix_mode=0, html=False,
-                 create_labels=False, asm_labels=True, min_address=0, max_address=65536, variables=(),
-                 fields=None, snapshot=None, expands=None):
+                 create_labels=False, asm_labels=True, label_fmt=None, min_address=0, max_address=65536,
+                 variables=(), fields=None, snapshot=None, expands=None):
         self.skoolfile = skoolfile
         self._assembler = get_assembler()
         self.utility = get_instruction_utility()
         self.mode = Mode(case, base, asm_mode & 3, warnings, fix_mode, html, create_labels, asm_labels, self._assembler)
+        self.label_fmt = label_fmt or ('L{address}', '{main}_{index}')
         self.case = case
         self.base = base
         if fields:
@@ -469,6 +473,7 @@ class SkoolParser:
             self.mode.html,
             self.mode.create_labels,
             self.mode.asm_labels,
+            self.label_fmt,
             fields=self.fields,
             snapshot=self.snapshot[:],
             expands=self.expands[:]
@@ -501,6 +506,19 @@ class SkoolParser:
             return instruction.get_addr_str()
         return self.mode.get_addr_str(address, default)
 
+    def get_instruction_timings(self, start, stop, asm_id=''):
+        timings = []
+        address = start
+        while address < stop:
+            instruction = self.get_instruction(address, asm_id)
+            if instruction and instruction.bytes:
+                timings.append((address, z80.get_timing(instruction)))
+                address += len(instruction.bytes)
+            else:
+                timings.append((address, None))
+                break
+        return timings
+
     def convert_equ_value(self, value):
         return self.mode.convert_int_str(value, '{}', '${0:02X}')
 
@@ -664,7 +682,11 @@ class SkoolParser:
             if self.mode.assemble:
                 self.mode.data.append(directive)
         elif directive.startswith('expand='):
-            self.expands.append(directive[7:])
+            value = directive[7:]
+            if value.startswith('+') and self.expands:
+                self.expands[-1] += value[1:]
+            else:
+                self.expands.append(value)
         elif directive.startswith('keep'):
             self.mode.keep = parse_asm_keep_directive(directive)
         elif directive.startswith('refs='):
@@ -673,6 +695,9 @@ class SkoolParser:
             self._parse_remote_directive(directive[7:])
         elif directive.startswith('replace='):
             self._add_replacement(directive[8:])
+        elif directive.startswith('rom'):
+            rom = read_bin_file(ROM48)
+            self.snapshot[:len(rom)] = rom
         elif directive.startswith('assemble='):
             html_value, asm_value = [parse_int(i) for i in (directive[9:] + ',').split(',')][:2]
             if self.mode.html and html_value is not None:
@@ -688,7 +713,7 @@ class SkoolParser:
             if directive.startswith(('isub=', 'ssub=', 'rsub=', 'ofix=', 'bfix=', 'rfix=')):
                 value = directive[5:].rstrip()
                 if value.startswith('!'):
-                    if self.mode.weights[directive[:4]]:
+                    if self.mode.weights[directive[:4]] > (0, 0):
                         removed.update(parse_address_range(value[1:]))
                 else:
                     self.mode.add_sub(directive[:4], value)
@@ -765,12 +790,14 @@ class SkoolParser:
             if instructions:
                 main_label = instructions[0].asm_label
                 if self.mode.create_labels and (not main_label or main_label == '*'):
-                    main_label = instructions[0].asm_label = 'L{0}'.format(entry.addr_str)
+                    main_label = instructions[0].asm_label = self.label_fmt[0].format(address=entry.addr_str, location=entry.address)
                 if main_label:
                     index = 0
                     for instruction in instructions[1:]:
                         if instruction.ctl == '*' and instruction.asm_label is None or instruction.asm_label == '*':
-                            instruction.asm_label = '{0}_{1}'.format(main_label, index)
+                            instruction.asm_label = self.label_fmt[1].format(
+                                address=instruction.addr_str, index=index, location=instruction.address, main=main_label
+                            )
                             index += 1
 
 class Mode:
@@ -791,19 +818,19 @@ class Mode:
         else:
             self.addr_fmt = '{0:04X}'
         self.weights = {
-            'isub': int(asm_mode > 0),
-            'ssub': 2 * int(asm_mode > 1),
-            'rsub': 3 * int(asm_mode > 2),
-            'ofix': 4 * int(fix_mode > 0),
-            'bfix': 5 * int(fix_mode > 1),
-            'rfix': 6 * int(fix_mode > 2)
+            'isub': (0, int(asm_mode > 0)),
+            'ssub': (0, 2 * int(asm_mode > 1)),
+            'rsub': (0, 3 * int(asm_mode > 2)),
+            'ofix': (int(fix_mode > 0), 0),
+            'bfix': (2 * int(fix_mode > 1), 0),
+            'rfix': (3 * int(fix_mode > 2), 0)
         }
         self.data = []
         self.reset()
 
     def reset(self):
         self.label = None
-        self.subs = defaultdict(list, {0: ()})
+        self.subs = defaultdict(list, {(0, 0): ()})
         self.keep = None
         self.nowarn = None
         self.refs = ((), ())
@@ -812,7 +839,7 @@ class Mode:
 
     def add_sub(self, directive, value):
         weight = self.weights[directive]
-        if weight:
+        if weight > (0, 0):
             self.subs[weight].append(value)
 
     def compose_instructions(self, subs, current=False):
@@ -932,6 +959,7 @@ class Mode:
         return int_str
 
 class InstructionUtility:
+    # Component API
     def convert(self, entries, base, case):
         """Convert the base and case of every instruction in a skool file.
 
@@ -1008,6 +1036,7 @@ class InstructionUtility:
                 operation = re.sub('(I[XY])L', r'\1l', operation)
         return operation
 
+    # Component API
     def substitute_labels(self, entries, remote_entries, labels, mode, warn):
         """Replace addresses with labels in the operands of every instruction
         in a skool file.
@@ -1093,6 +1122,7 @@ class InstructionUtility:
             # instruction (use @keep or @nowarn if this is OK)
             self._warn('Unreplaced address ({})'.format(addr_str), instruction, address)
 
+    # Component API
     def calculate_references(self, entries, remote_entries):
         """
         Generate a dictionary of references (for each instruction that refers
@@ -1127,6 +1157,7 @@ class InstructionUtility:
                         referrers[instruction].add(entry_map[ref_addr])
         return references, referrers
 
+    # Component API
     def set_byte_values(self, instruction, assemble):
         """Decide whether to set byte values in the memory snapshot and for an
         instruction.
