@@ -1,4 +1,4 @@
-# Copyright 2013-2017, 2019, 2020 Richard Dymond (rjdymond@gmail.com)
+# Copyright 2013-2017, 2019-2021 Richard Dymond (rjdymond@gmail.com)
 #
 # This file is part of SkoolKit.
 #
@@ -67,8 +67,8 @@ class Registers:
 
 ###############################################################################
 
-# http://www.worldofspectrum.org/faq/reference/z80format.htm
-# http://www.worldofspectrum.org/faq/reference/128kreference.htm
+# https://worldofspectrum.net/features/faq/reference/z80format.htm
+# https://worldofspectrum.net/features/faq/reference/128kreference.htm
 
 BLOCK_ADDRESSES_48K = {
     4: '32768-49151 8000-BFFF',
@@ -204,7 +204,7 @@ def _analyse_z80(z80file):
 
 ###############################################################################
 
-# http://www.spectaculator.com/docs/zx-state/intro.shtml
+# https://www.spectaculator.com/docs/zx-state/intro.shtml
 
 SZX_MACHINES = {
     0: '16K ZX Spectrum',
@@ -303,7 +303,7 @@ def _analyse_szx(szxfile):
 
 ###############################################################################
 
-# http://www.worldofspectrum.org/faq/reference/formats.htm#SNA
+# https://worldofspectrum.net/features/faq/reference/formats.htm#SNA
 
 def _print_ram_banks(sna):
     bank = sna[49181] & 7
@@ -454,18 +454,18 @@ def _find_text(snapshot, text):
         if snapshot[a:a + size] == byte_values:
             print("{0}-{1} {0:04X}-{1:04X}: {2}".format(a, a + size - 1, text))
 
-def _peek(snapshot, specs):
+def _peek(snapshot, specs, fmt):
     for addr1, addr2, step in _get_address_ranges(specs):
         for a in range(addr1, addr2 + 1, step):
             value = snapshot[a]
             char = get_char(value, '', 'UDG-{}', True)
-            print('{0:>5} {0:04X}: {1:>3}  {1:02X}  {1:08b}  {2}'.format(a, value, char))
+            print(fmt.format(address=a, value=value, char=char))
 
-def _word(snapshot, specs):
+def _word(snapshot, specs, fmt):
     for addr1, addr2, step in _get_address_ranges(specs, 2):
         for a in range(addr1, addr2 + 1, step):
             value = snapshot[a] + 256 * snapshot[a + 1]
-            print('{0:>5} {0:04X}: {1:>5}  {1:04X}'.format(a, value))
+            print(fmt.format(address=a, value=value))
 
 def run(infile, options, config):
     if any((options.find, options.tile, options.text, options.call_graph, options.peek,
@@ -480,9 +480,9 @@ def run(infile, options, config):
         elif options.call_graph:
             _call_graph(snapshot, options.ctlfiles, infile, start, end, config)
         elif options.peek:
-            _peek(snapshot, options.peek)
+            _peek(snapshot, options.peek, config['Peek'])
         elif options.word:
-            _word(snapshot, options.word)
+            _word(snapshot, options.word, config['Word'])
         else:
             if options.basic:
                 print(BasicLister().list_basic(snapshot))
@@ -508,9 +508,9 @@ def main(args):
     group = parser.add_argument_group('Options')
     group.add_argument('-b', '--basic', action='store_true',
                        help='List the BASIC program.')
-    group.add_argument('-c', '--ctl', dest='ctlfiles', metavar='FILE', action='append', default=[],
-                       help="Use FILE as a control file when generating a call graph. FILE may be '-' for standard input. "
-                            "This option may be used multiple times.")
+    group.add_argument('-c', '--ctl', dest='ctlfiles', metavar='PATH', action='append', default=[],
+                       help="When generating a call graph, specify a control file to use, or a directory from which to read control files. "
+                            "PATH may be '-' for standard input. This option may be used multiple times.")
     group.add_argument('-f', '--find', metavar='A[,B...[-M[-N]]]',
                        help='Search for the byte sequence A,B... with distance ranging from M to N (default=1) between bytes.')
     group.add_argument('-g', '--call-graph', action='store_true',
