@@ -35,15 +35,17 @@ def create_header_block(title='', start=0, length=0, data_type=3):
     header.extend([ord(c) for c in title[:10].ljust(10)])
     header.extend((length % 256, length // 256))
     header.extend((start % 256, start // 256))
-    header.extend((0, 0))
+    if data_type == 0:
+        header.extend((length % 256, length // 256))
+    else:
+        header.extend((0, 0))
     header.append(get_parity(header))
     return header
 
 def create_data_block(data):
-    parity = 0
-    for b in data:
-        parity ^= b
-    return [255] + data + [get_parity(data)]
+    data_block = [255] + data
+    data_block.append(get_parity(data_block))
+    return data_block
 
 def create_tap_data_block(data):
     data_block = create_data_block(data)
@@ -62,10 +64,10 @@ def create_tzx_data_block(data):
     block.extend(data_block)
     return block
 
-def create_tzx_header_block(title='', start=0, data_type=3):
+def create_tzx_header_block(title='', start=0, length=0, data_type=3):
     block = [16] # Block ID
     block.extend((0, 0)) # Pause duration
-    data_block = create_header_block(title, start, data_type=data_type)
+    data_block = create_header_block(title, start, length, data_type)
     length = len(data_block)
     block.extend((length % 256, length // 256))
     block.extend(data_block)
@@ -75,6 +77,7 @@ class Stream:
     def __init__(self, binary=False):
         self._buffer = BytesIO() if binary else StringIO()
         self.buffer = self
+        self.name = ''
 
     def __enter__(self):
         return self
@@ -91,10 +94,16 @@ class Stream:
     def getvalue(self):
         return self._buffer.getvalue()
 
+    def tell(self):
+        return self._buffer.tell()
+
     def clear(self):
         self._buffer.seek(0)
         self._buffer.truncate()
 
+    def close(self):
+        return
+
 class StdIn:
     def __init__(self, data):
         self.data = data
