@@ -87,8 +87,8 @@ ATTR_INDEX = create_attr_index()
 
 class ImageWriterOptionsTest(SkoolKitTestCase):
     def test_change_option_values(self):
-        options = {PNG_COMPRESSION_LEVEL: 3}
-        image_writer = ImageWriter(options=options)
+        config = {PNG_COMPRESSION_LEVEL: 3}
+        image_writer = ImageWriter(config)
         self.assertEqual(image_writer.options[PNG_COMPRESSION_LEVEL], 3)
 
     def test_default_option_values(self):
@@ -98,7 +98,7 @@ class ImageWriterOptionsTest(SkoolKitTestCase):
         self.assertEqual(image_writer.options[PNG_ALPHA], 255)
 
     def test_invalid_option_value(self):
-        image_writer = ImageWriter(options={PNG_COMPRESSION_LEVEL: 'NaN'})
+        image_writer = ImageWriter({PNG_COMPRESSION_LEVEL: 'NaN'})
         self.assertEqual(image_writer.options[PNG_COMPRESSION_LEVEL], 9)
 
 class ImageWriterTest:
@@ -108,8 +108,7 @@ class ImageWriterTest:
     def _get_dword(self, stream, index):
         return index + 4, 16777216 * stream[index] + 65536 * stream[index + 1] + 256 * stream[index + 2] + stream[index + 3]
 
-    def _get_image_data(self, image_writer, udg_array, scale=1, mask=0, tindex=0, alpha=-1, x=0, y=0, width=None, height=None):
-        frame = Frame(udg_array, scale, mask, x, y, width, height, tindex=tindex, alpha=alpha)
+    def _get_image_data(self, image_writer, frame):
         img_stream = BytesIO()
         image_writer.write_image([frame], img_stream)
         img_bytes = [b for b in img_stream.getvalue()]
@@ -716,7 +715,7 @@ class ImageWriterTest:
 
     def test_mask1_alpha(self):
         # OR-AND mask, alpha < 255
-        iw_args = {'options': self.alpha_option}
+        iw_args = {'config': self.alpha_option}
         udg = Udg(88, (34,) * 8, (163,) * 8)
         udg_array = [[udg]]
         self._test_image(udg_array, mask=1, iw_args=iw_args)
@@ -749,42 +748,42 @@ class ImageWriterTest:
 
     def test_mask1_flashing_no_animation(self):
         # OR-AND mask, flashing, no animation
-        iw_args = {'options': {self.animation_flag: 0}}
+        iw_args = {'config': {self.animation_flag: 0}}
         udg = Udg(184, (240,) * 8, (243,) * 8)
         udg_array = [[udg]]
         self._test_image(udg_array, mask=1, iw_args=iw_args)
 
     def test_alpha_no_mask(self):
         # No mask, alpha < 255
-        iw_args = {'options': self.alpha_option}
+        iw_args = {'config': self.alpha_option}
         udg = Udg(56, (132,) * 8)
         udg_array = [[udg]]
         self._test_image(udg_array, iw_args=iw_args)
 
     def test_flashing_no_animation(self):
         # No mask, flashing, no animation
-        iw_args = {'options': {self.animation_flag: 0}}
+        iw_args = {'config': {self.animation_flag: 0}}
         udg = Udg(184, (240,) * 8)
         udg_array = [[udg]]
         self._test_image(udg_array, iw_args=iw_args)
 
     def test_alternative_transparent_colour_paper(self):
         # White (PAPER) as transparent colour
-        iw_args = {'options': self.alpha_option}
+        iw_args = {'config': self.alpha_option}
         udg = Udg(56, (15,) * 8)
         udg_array = [[udg]]
         self._test_image(udg_array, tindex=8, iw_args=iw_args)
 
     def test_alternative_transparent_colour_ink(self):
         # Black (INK) as transparent colour
-        iw_args = {'options': self.alpha_option}
+        iw_args = {'config': self.alpha_option}
         udg = Udg(56, (15,) * 8)
         udg_array = [[udg]]
         self._test_image(udg_array, tindex=1, iw_args=iw_args)
 
     def test_alternative_transparent_colour_overridden_by_mask(self):
         # White (PAPER) as transparent colour, overridden by mask
-        iw_args = {'options': self.alpha_option}
+        iw_args = {'config': self.alpha_option}
         udg = Udg(56, (15,) * 8, (207,) * 8)
         udg_array = [[udg]]
         self._test_image(udg_array, mask=1, tindex=8, iw_args=iw_args)
@@ -818,6 +817,13 @@ class ImageWriterTest:
         exp_pixels = [[salmon, salmon, indigo, indigo, indigo, indigo, salmon, salmon]] * 8
         self._test_image(udg_array, iw_args=iw_args, exp_pixels=exp_pixels)
 
+    def test_frame_colour_change_between_writes(self):
+        udg = Udg(56, [0] * 8)
+        frame = Frame([[udg]])  # All white pixels
+        self._test_image(frame)
+        udg.data[0] = 255       # Add black pixels
+        self._test_image(frame)
+
     def test_animation(self):
         # 3 frames, 2 colours, 16x8
         frame1 = Frame([[Udg(6, (128,) * 8), Udg(6, (0,) * 8)]], delay=20)
@@ -826,6 +832,14 @@ class ImageWriterTest:
         frames = [frame1, frame2, frame3]
         self._test_animated_image(frames)
 
+    def test_animation_with_offsets(self):
+        # 3 frames: 16x16; 8x8 at (2,3); 8x8 at (5,4)
+        frame1 = Frame([[Udg(56, (128,) * 8)]], 2)
+        frame2 = Frame([[Udg(56, (64,) * 8)]], x_offset=2, y_offset=3)
+        frame3 = Frame([[Udg(56, (32,) * 8)]], x_offset=5, y_offset=4)
+        frames = [frame1, frame2, frame3]
+        self._test_animated_image(frames)
+
     def test_animation_cropped(self):
         # 2 frames, 4 colours, 4x4
         frame1 = Frame([[Udg(56, (128,) * 8)]], x=1, y=1, width=4, height=4)
@@ -835,7 +849,7 @@ class ImageWriterTest:
 
     def test_animation_mask1(self):
         # 2 frames, transparency on frame 2
-        iw_args = {'options': self.alpha_option}
+        iw_args = {'config': self.alpha_option}
         frame1 = Frame([[Udg(49, (64,) * 8)]])
         frame2 = Frame([[Udg(184, (240,) * 8, (243,) * 8)]], mask=1)
         frames = [frame1, frame2]
@@ -851,7 +865,7 @@ class ImageWriterTest:
 
     def test_animation_with_alternative_transparent_colour_on_first_frame_only(self):
         # White (PAPER) as transparent colour on first frame
-        iw_args = {'options': self.alpha_option}
+        iw_args = {'config': self.alpha_option}
         frame1 = Frame([[Udg(56, (15,) * 8)]], tindex=8)
         frame2 = Frame([[Udg(1, (15,) * 8)]])
         frames = [frame1, frame2]
@@ -859,7 +873,7 @@ class ImageWriterTest:
 
     def test_animation_with_alternative_transparent_colour_on_second_frame_only(self):
         # Yellow (PAPER) as transparent colour on second frame
-        iw_args = {'options': self.alpha_option}
+        iw_args = {'config': self.alpha_option}
         frame1 = Frame([[Udg(56, (15,) * 8)]], tindex=7)
         frame2 = Frame([[Udg(48, (15,) * 8)]])
         frames = [frame1, frame2]
@@ -868,7 +882,7 @@ class ImageWriterTest:
     def test_animation_with_alternative_transparent_colour_overridden_by_mask(self):
         # White (PAPER) as transparent colour on first frame, overridden by
         # mask on second frame
-        iw_args = {'options': self.alpha_option}
+        iw_args = {'config': self.alpha_option}
         frame1 = Frame([[Udg(56, (15,) * 8)]], tindex=8)
         frame2 = Frame([[Udg(56, (15,) * 8, (207,) * 8)]], mask=1)
         frames = [frame1, frame2]
@@ -1083,8 +1097,13 @@ class PngWriterTest(SkoolKitTestCase, ImageWriterTest):
         return i
 
     def _test_image(self, udg_array, scale=1, mask=0, tindex=0, alpha=-1, x=0, y=0, width=None, height=None, iw_args=None, exp_pixels=None):
+        if isinstance(udg_array, Frame):
+            frame = udg_array
+            udg_array = frame.udgs
+        else:
+            frame = Frame(udg_array, scale, mask, x, y, width, height, tindex=tindex, alpha=alpha)
         image_writer = ImageWriter(**(iw_args or {}))
-        img_bytes = self._get_image_data(image_writer, udg_array, scale, mask, tindex, alpha, x, y, width, height)
+        img_bytes = self._get_image_data(image_writer, frame)
 
         exp_pixels2 = None
         if exp_pixels is None:
@@ -1157,7 +1176,7 @@ class PngWriterTest(SkoolKitTestCase, ImageWriterTest):
             frame_palette, f_has_trans, f_has_tindex, pixels, pixels2, frame2_xy = self._get_pixels_from_udg_array(frame.udgs, frame.scale, frame.mask, exp_tindex, x, y, width, height)
             has_trans = has_trans or f_has_trans
             has_tindex = has_tindex or f_has_tindex
-            frame_data.append((width, height, pixels, frame.delay))
+            frame_data.append((width, height, pixels, frame.delay, frame.x_offset, frame.y_offset))
             for c in frame_palette:
                 if c not in exp_palette:
                     exp_palette.append(c)
@@ -1197,8 +1216,8 @@ class PngWriterTest(SkoolKitTestCase, ImageWriterTest):
 
         # Frames
         seq_num = 0
-        for exp_width, exp_height, exp_pixels, exp_delay in frame_data:
-            i = self._check_fctl(img_bytes, i, seq_num, exp_width, exp_height, exp_delay=exp_delay)
+        for exp_width, exp_height, exp_pixels, exp_delay, exp_x_offset, exp_y_offset in frame_data:
+            i = self._check_fctl(img_bytes, i, seq_num, exp_width, exp_height, exp_x_offset, exp_y_offset, exp_delay)
             seq_num += 1
             if seq_num == 1:
                 i = self._check_idat(img_bytes, i, exp_bit_depth, palette, exp_pixels, exp_width)
