@@ -18,7 +18,7 @@ def mock_config(name):
 
 class MockSkoolParser:
     def __init__(self, skoolfile, case, base, asm_mode, warnings, fix_mode, html,
-                 create_labels, asm_labels, min_address, max_address, variables):
+                 create_labels, asm_labels, label_fmt, min_address, max_address, variables):
         global mock_skool_parser
         mock_skool_parser = self
         self.skoolfile = skoolfile
@@ -30,18 +30,20 @@ class MockSkoolParser:
         self.html = html
         self.create_labels = create_labels
         self.asm_labels = asm_labels
+        self.label_fmt = label_fmt
         self.min_address = min_address
         self.max_address = max_address
         self.properties = {}
         self.asm_writer_class = ''
 
 class MockAsmWriter:
-    def __init__(self, parser, properties, templates):
+    def __init__(self, parser, properties, templates, config):
         global mock_asm_writer
         mock_asm_writer = self
         self.parser = parser
         self.properties = properties
         self.templates = templates
+        self.config = config
         self.wrote = False
 
     def write(self):
@@ -62,7 +64,7 @@ class Skool2AsmTest(SkoolKitTestCase):
     def test_default_option_values(self):
         skoolfile = 'test.skool'
         skool2asm.main((skoolfile,))
-        fname, options = run_args
+        fname, options, config = run_args
         self.assertEqual(fname, skoolfile)
         self.assertFalse(options.quiet)
         self.assertIsNone(options.writer)
@@ -78,7 +80,9 @@ class Skool2AsmTest(SkoolKitTestCase):
         self.assertEqual(options.params, [])
         self.assertEqual(options.variables, [])
         self.assertFalse(options.force)
-        self.assertEqual(options.templates, '')
+        self.assertEqual(config['Templates'], '')
+        self.assertEqual(config['EntryLabel'], 'L{address}')
+        self.assertEqual(config['EntryPointLabel'], '{main}_{index}')
 
     @patch.object(skool2asm, 'SkoolParser', MockSkoolParser)
     @patch.object(skool2asm, 'AsmWriter', MockAsmWriter)
@@ -96,6 +100,7 @@ class Skool2AsmTest(SkoolKitTestCase):
         self.assertFalse(mock_skool_parser.html)
         self.assertFalse(mock_skool_parser.create_labels)
         self.assertTrue(mock_skool_parser.asm_labels)
+        self.assertEqual(mock_skool_parser.label_fmt, ('L{address}', '{main}_{index}'))
         self.assertEqual(mock_skool_parser.min_address, 0)
         self.assertEqual(mock_skool_parser.max_address, 65536)
 
@@ -111,6 +116,8 @@ class Skool2AsmTest(SkoolKitTestCase):
             Base=16
             Case=1
             CreateLabels=1
+            EntryLabel=L_{address}
+            EntryPointLabel={main}__{index}
             Quiet=1
             Set-bullet=-
             Set-indent=4
@@ -120,7 +127,7 @@ class Skool2AsmTest(SkoolKitTestCase):
         self.write_text_file(dedent(ini).strip(), 'skoolkit.ini')
         skoolfile = 'test.skool'
         skool2asm.main((skoolfile,))
-        fname, options = run_args
+        fname, options, config = run_args
         self.assertEqual(fname, skoolfile)
         self.assertTrue(options.quiet)
         self.assertIsNone(options.writer)
@@ -128,12 +135,55 @@ class Skool2AsmTest(SkoolKitTestCase):
         self.assertEqual(options.base, 16)
         self.assertEqual(options.asm_mode, 1)
         self.assertFalse(options.warn)
-        self.assertEqual(options.templates, 'templates.ini')
         self.assertEqual(options.fix_mode, 0)
         self.assertTrue(options.create_labels)
         self.assertEqual(options.start, 0)
         self.assertEqual(options.end, 65536)
         self.assertEqual(sorted(options.properties), ['bullet=-', 'indent=4'])
+        self.assertEqual(config['Templates'], 'templates.ini')
+        self.assertEqual(config['EntryLabel'], 'L_{address}')
+        self.assertEqual(config['EntryPointLabel'], '{main}__{index}')
+
+    @patch.object(skool2asm, 'SkoolParser', MockSkoolParser)
+    @patch.object(skool2asm, 'AsmWriter', MockAsmWriter)
+    def test_config_read_from_file_is_passed(self):
+        templates = """
+            [comment]
+            ;; {text}
+        """
+        ini = """
+            [skool2asm]
+            Base=16
+            Case=1
+            CreateLabels=1
+            EntryLabel=L_{{address}}
+            EntryPointLabel={{main}}__{{index}}
+            Set-bullet=-
+            Set-indent=4
+            Templates={}
+            Warnings=0
+        """.format(self.write_text_file(dedent(templates).strip()))
+        self.write_text_file(dedent(ini).strip(), 'skoolkit.ini')
+        skoolfile = 'test.skool'
+        skool2asm.main((skoolfile,))
+
+        self.assertEqual(mock_skool_parser.skoolfile, skoolfile)
+        self.assertEqual(mock_skool_parser.case, 1)
+        self.assertEqual(mock_skool_parser.base, 16)
+        self.assertEqual(mock_skool_parser.asm_mode, 1)
+        self.assertFalse(mock_skool_parser.warnings)
+        self.assertEqual(mock_skool_parser.fix_mode, 0)
+        self.assertFalse(mock_skool_parser.html)
+        self.assertTrue(mock_skool_parser.create_labels)
+        self.assertTrue(mock_skool_parser.asm_labels)
+        self.assertEqual(mock_skool_parser.label_fmt, ('L_{address}', '{main}__{index}'))
+        self.assertEqual(mock_skool_parser.min_address, 0)
+        self.assertEqual(mock_skool_parser.max_address, 65536)
+
+        self.assertIs(mock_asm_writer.parser, mock_skool_parser)
+        self.assertEqual({'bullet': '-', 'indent': '4', 'warnings': '0'}, mock_asm_writer.properties)
+        self.assertEqual({'comment': ';; {text}'}, mock_asm_writer.templates)
+        self.assertTrue(mock_asm_writer.wrote)
 
     @patch.object(skool2asm, 'run', mock_run)
     def test_invalid_option_values_read_from_file(self):
@@ -146,7 +196,7 @@ class Skool2AsmTest(SkoolKitTestCase):
         self.write_text_file(dedent(ini).strip(), 'skoolkit.ini')
         skoolfile = 'test.skool'
         skool2asm.main((skoolfile,))
-        fname, options = run_args
+        fname, options, config = run_args
         self.assertEqual(fname, skoolfile)
         self.assertFalse(options.quiet)
         self.assertIsNone(options.writer)
@@ -536,9 +586,12 @@ class Skool2AsmTest(SkoolKitTestCase):
         self.assertEqual(error, '')
         exp_output = """
             [skool2asm]
+            Address=
             Base=0
             Case=0
             CreateLabels=0
+            EntryLabel=L{address}
+            EntryPointLabel={main}_{index}
             Quiet=0
             Templates=
             Warnings=1
@@ -557,9 +610,12 @@ class Skool2AsmTest(SkoolKitTestCase):
         self.assertEqual(error, '')
         exp_output = """
             [skool2asm]
+            Address=
             Base=10
             Case=1
             CreateLabels=0
+            EntryLabel=L{address}
+            EntryPointLabel={main}_{index}
             Quiet=1
             Templates=
             Warnings=1
