@@ -1,8 +1,9 @@
+import os.path
 from textwrap import dedent
 from unittest.mock import patch
 
 from skoolkittest import SkoolKitTestCase
-from skoolkit import SkoolKitError, snapinfo, VERSION
+from skoolkit import SkoolKitError, components, snapinfo, VERSION
 from skoolkit.config import COMMANDS
 
 def mock_run(*args):
@@ -916,6 +917,33 @@ class SnapinfoTest(SkoolKitTestCase):
         ctlfiles = [self.write_text_file(dedent(c).strip(), suffix='.ctl') for c in (ctl1, ctl2)]
         self._test_sna(ram, exp_output, '-g -c {} --ctl {}'.format(*ctlfiles), ctlfiles=ctlfiles)
 
+    def test_option_c_with_directory(self):
+        ram = [201, 195, 0, 64] + [0] * 49148
+        ctl1 = """
+            @ 16384 label=END
+            c 16384
+        """
+        ctl2 = """
+            @ 16385 label=START
+            c 16385
+            i 16388
+        """
+        exp_output = r"""
+            // Unconnected: None
+            // Orphans: 16385
+            // First instruction not used: None
+            digraph {
+            node [shape=record]
+            16384 [label="16384 4000\nEND"]
+            16385 [label="16385 4001\nSTART"]
+            16385 -> {16384}
+            }
+        """
+        ctldir = self.make_directory()
+        ctl1f = self.write_text_file(dedent(ctl1).strip(), path=os.path.join(ctldir, 'bar.ctl'))
+        ctl2f = self.write_text_file(dedent(ctl2).strip(), path=os.path.join(ctldir, 'foo.ctl'))
+        self._test_sna(ram, exp_output, '-g -c {}'.format(ctldir), ctlfiles=(ctl1f, ctl2f))
+
     def test_option_f_with_single_byte(self):
         ram = [0] * 49152
         address = 53267
@@ -1009,6 +1037,22 @@ class SnapinfoTest(SkoolKitTestCase):
         """
         self._test_sna(ram, exp_output, '-g')
 
+    @patch.object(components, 'SK_CONFIG', None)
+    def test_option_g_with_no_ctl_file_and_custom_default_disassembly_start_address(self):
+        ini = "[skoolkit]\nDefaultDisassemblyStartAddress=64222"
+        self.write_text_file(ini, 'skoolkit.ini')
+        ram = [0] * 49152
+        exp_output = r"""
+            // Unconnected: 64222
+            // Orphans: None
+            // First instruction not used: None
+            digraph {
+            node [shape=record]
+            64222 [label="64222 FADE\n"]
+            }
+        """
+        self._test_sna(ram, exp_output, '-g')
+
     def test_option_call_graph_with_ctl_file(self):
         ram = [195, 3, 64, 201] + [0] * 49148
         ctl = """
@@ -1637,6 +1681,8 @@ class SnapinfoTest(SkoolKitTestCase):
             NodeAttributes=shape=record
             NodeId={address}
             NodeLabel="{address} {address:04X}\n{label}"
+            Peek={address:>5} {address:04X}: {value:>3}  {value:02X}  {value:08b}  {char}
+            Word={address:>5} {address:04X}: {value:>5}  {value:04X}
         """
         self.assertEqual(dedent(exp_output).strip(), output.rstrip())
 
@@ -1645,6 +1691,7 @@ class SnapinfoTest(SkoolKitTestCase):
             [snapinfo]
             NodeAttributes=shape=box
             NodeLabel="{label}"
+            Peek=${address:04X},${value:02X}
         """
         self.write_text_file(dedent(ini).strip(), 'skoolkit.ini')
         output, error = self.run_snapinfo('--show-config', catch_exit=0)
@@ -1656,6 +1703,8 @@ class SnapinfoTest(SkoolKitTestCase):
             NodeAttributes=shape=box
             NodeId={address}
             NodeLabel="{label}"
+            Peek=${address:04X},${value:02X}
+            Word={address:>5} {address:04X}: {value:>5}  {value:04X}
         """
         self.assertEqual(dedent(exp_output).strip(), output.rstrip())
 
@@ -2046,3 +2095,23 @@ class SnapinfoTest(SkoolKitTestCase):
             }
         """
         self._test_sna(ram, exp_output, '-g', ctl)
+
+    def test_config_Peek(self):
+        ram = [0] * 49152
+        ram[33616:33620] = [1, 65, 144, 165]
+        exp_output = """
+            $c350:001()
+            $c351:065(A)
+            $c352:144(UDG-A)
+            $c353:165(RND)
+        """
+        self._test_sna(ram, exp_output, '-p 50000-50003 -I Peek=${address:04x}:{value:03d}({char})')
+
+    def test_config_Word(self):
+        ram = [0] * 49152
+        ram[43616:43620] = [1, 2, 254, 255]
+        exp_output = """
+            $EA60,$0201
+            $EA62,$FFFE
+        """
+        self._test_sna(ram, exp_output, '-w 60000-60003 -I Word=${address:04X},${value:04X}')
